// Gameballworld.cpp: implementation of the GameBallworld class and associated classes.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "gameballworld.h"
#include "force.h"
#include "listener.h"
#include "pop.h" //for playSound
#include "spritepolygon.h" 
#include "critterviewer.h"
#include "graphicsopengl.h"
#include "graphicsmfc.h"

IMPLEMENT_SERIAL( cGameBallworld, cGame, 0);
IMPLEMENT_SERIAL( cCritterBallProp, cCritter, 0);
IMPLEMENT_SERIAL( cCritterBallworldPlayer, cCritterArmedPlayer, 0);
IMPLEMENT_SERIAL( cCritterTreasure, cCritter, 0);

//==============================cGameBallworld=======
Real cGameBallworld::TREASURERADIUS = 1.2;

cGameBallworld::cGameBallworld()
{
//Fix the menu selections you'll allow.
	_menuflags &= ~cGame::MENU_BOUNCEWRAP; 
		/* Default _menuflags from cGame has autoplay off, hopper off. 
		Turn off wrap because it's bad with gravity, we want balls bouncing on bottom.*/
	_menuflags |= cGame::MENU_HOPPER; //Turn on hopper.
	//_arrayHCURSOR has only the _hCursorArrow inherited from cGame.
	_spritetype = cGame::ST_BUBBLES;
	_arrayHCURSOR.Add(((CPopApp*)::AfxGetApp())->_hCursorPlay); //Shooting cursor.
	_border.set(100.0, 12.0, 0.0); //Use a long, thin world.
	setBackgroundBitmap(IDB_BACKGROUND); /*  Sets the cSpriteBackground for the background bitmap.
		Note that you must call this after the _border has its size fully initialized. So each
		cGame child with a different border should call this again in its constructor. */
	setWrapflag(cCritter::BOUNCE);
	_seedcount = 7; 
  	_arrayHCURSOR.Add(((CPopApp*)::AfxGetApp())->_hCursorPlay);
  	_arrayHCURSOR.Add(((CPopApp*)::AfxGetApp())->_hCursorZap);
  	_arrayHCURSOR.Add(((CPopApp*)::AfxGetApp())->_hCursorReplicate);
	setPlayer(new cCritterBallworldPlayer(this));
	_ptreasure = new cCritterTreasure(this);
}

void cGameBallworld::initializeView(CPopView *pview)
{
	cGame::initializeView(pview);
//	pview->setGraphicsClass(RUNTIME_CLASS(cGraphicsOpenGL)); //Comment this in for a 3D start
	pview->pviewpointcritter()->setTrackplayer(TRUE); //Always look at the player.

}

void cGameBallworld::initializeViewpoint(cCritterViewer *pviewer)
{
	if (pviewer->is3D())
	{
		pviewer->setViewpoint(cVector3(0.0, -0.2, 1.0), //pplayer()->position());
			cVector(border().lox(), border().midy()));
			//The setViewpoint args are the (directiontoviewer, lookatpoint);
		pviewer->zoom(7.0); //The bigger the world the more zoom you need.
	}
	else //2D case
	{
		pviewer->setViewpoint(cVector::ZAXIS, //pplayer()->position());
			cVector(border().lox(), border().midy()));
		pviewer->zoom(4.0);
	}
}

void cGameBallworld::seedCritters()
{
	pbiota()->purgeCritters(RUNTIME_CLASS(cCritterBullet));
	pbiota()->purgeCritters(RUNTIME_CLASS(cCritterBallProp));
	for (int i=0; i<_seedcount; i++)
		new cCritterBallProp(this);
	pplayer()->moveTo(border().locorner());
}

void cGameBallworld::adjustGameParameters()
{
// (1) End the game if the player is dead
	if (!health()  && !_gameover) //Player's been killed and game's not over.
	{
		_gameover = TRUE;
		pplayer()->addScore(_scorecorrection); // So user can reach _maxscore 
		playSound("Tada");
		return; 
	}
// (2) Also don't let the ballcount diminish.
			//(need to recheck propcount in case we just called seedCritters).
	int ballcount = pbiota()->count(RUNTIME_CLASS(cCritterBallProp));
	int ballstoadd = _seedcount - ballcount;
 	for (int i=0; i < ballstoadd; i++)
		new cCritterBallProp(this);
// (3) Maybe check some other conditions.
 }

CString cGameBallworld::statusMessage()
{
	CString cStrStatusBar;
	int nUpdatesPerSecond;
	CString cStrUpdatespersecond;
	CString cStrCount;
	CString cStrScore;
	CString cStrHealth;

	if (!gamepaused())
	{
		nUpdatesPerSecond = int(((CPopApp*)::AfxGetApp())->_timer.updatesPerSecond());
		if (!nUpdatesPerSecond)
			cStrUpdatespersecond.Format("Less than one update per second.");
		else	
			cStrUpdatespersecond.Format("Updates per second: %d.", nUpdatesPerSecond);
	}
	else
			cStrUpdatespersecond.Format("Animation is paused.");
	cStrScore.Format("Score: %d.", score());
	cStrHealth.Format("Health: %d.", health());
	int crittercount = _pbiota->count(RUNTIME_CLASS(cCritter));
	int bulletcount = _pbiota->count(RUNTIME_CLASS(cCritterBullet));
	cStrCount.Format("Ball Critters: %d.", crittercount-bulletcount-2); //Subtract two for player and treasure
	cStrStatusBar = cStrHealth + cStrScore + "  " + cStrCount + "  " + cStrUpdatespersecond; 
	return cStrStatusBar;
}

void cGameBallworld::Serialize(CArchive& ar) /* This is a bit of work, as I have to match
	_ppuck , _phockeyrobot , _pmygoal , _probotgoal to the pointers in the cBiota array. */
{
	int treasureindex;
	cGame::Serialize(ar);
	if (ar.IsStoring()) // Save 
 	{
		ar << _index(_ptreasure);
	}
	else //Load
	{
		ar >> treasureindex;
			// I have to cast from cCritter*
		_ptreasure = (cCritterTreasure*)(_pbiota->GetAt(treasureindex));
	}
}
//====================Critters for the Ballworld Game ==========================
//==================================================================


//==================cCritterBallworldPlayer=============

cCritterBallworldPlayer::cCritterBallworldPlayer(cGame *pownergame):
cCritterArmedPlayer(pownergame)
{
	setHealth(10);
	moveTo(_movebox.locorner());
//	setWrapflag(cCritter::BOUNCE);
	setWrapflag(cCritter::CLAMP); //Use CLAMP so you want to stop dead at edges.
	setBulletClass(RUNTIME_CLASS(cCritterBulletRubber)); //Maybe rubber bullets?
	setArmed(FALSE);  //Aw, let's don't use bullets.
//	setListener(new cListenerCursor());  //Use cListenerCursor for easier game
	setMaxspeed(40.0); //Need good speed to overcome gravity.
	setListener(new cListenerHopper(6.0, 0.2)); /* The two arguments specify
		the walkspeed and the hopuptime.  Walkspeed is how fast the left/right
		(and PgUp/PgDn if in 3D where the ground is xz plane ) atrow keys move 
		the critter, and the hopuptime is how many seconds you can accelerate up
		with the UP arrow. */
	setListenerAcceleration(160.0); //So Hopper can overcome gravity
	addForce(new cForceGravity(50.0)); /* Uses  gravity. Default strength is 25.0.
		Gravity	will affect player using cListenerHopper. */
//	addForce(new cForceDrag(0.5));
		 /* default friction strength is 0.5. */
	setRadius(0.5); //Default cCritter::PLAYERRADIUS is 0.4. 
//	setDensity(8.0 * density()); //To make it fall fast	
}

BOOL cCritterBallworldPlayer::collide(cCritter *pcritter)
{
	BOOL playerhigherthancritter = position().y() - radius() > pcritter->position().y();
/* If you are "higher" than the pcritter, as in jumping on it, you get a poitn
and the critter dies.  If you are lower than it, you lose health and the
critter also dies. To be higher, let's say your low point has to higher
than the critter's center. We compute playerhigherthancritter before the collide,
as collide can change the positions. */
	BOOL collided = cCritter::collide(pcritter);
	if (!collided)
		return FALSE;
/* If you're here, you collided.  We'll treat all the guys the same -- the collision
 with a Treasure is different, but we let the Treasure contol that collision. */
	if (playerhigherthancritter)
	{
		playSound("Ding");
		addScore(10);
	}
	else
		damage(1);
	pcritter->die();
	return TRUE;
}

//==================cCritterBallProp=============

cCritterBallProp::cCritterBallProp(cGame *pownergame):
cCritter(pownergame)
{
	if (pgame())
		setSprite(pgame()->randomSprite(pgame()->spritetype()));
	addForce(new cForceGravity(25.0, cVector( -0.03, -1, 0.0))); /* Make the
		gravity tilt towards the left to move them along. */
	addForce(new cForceDrag()); //Uses default friction strength 0.5.
	setDensity(2.0); 
	setMaxspeed(20.0);
   	randomize(cCritter::MF_VELOCITY | cSprite::MF_RADIUS);
	//setBounciness(0.9); //Not 1.0 means it loses a bit of energy with each bounce.
	setMinTwitchThresholdSpeed(4.0); //Means sprite doesn't switch direction unless it's moving fast
	Real playerx = 0.0;
	if (pownergame)
		playerx = pownergame->pplayer()->position().x();
	randomizePosition(cRealBox(cVector(playerx + 3.0,_movebox.loy()),
		_movebox.hicorner())); 
		/* I put them to the right of the player  */
}

void cCritterBallProp::die()
{
	pplayer()->addScore(value());
	cCritter::die();
}

void cCritterBallProp::update(CPopView *pactiveview)
{
	cCritter::update(pactiveview); //Always call this first
	if (_outcode & BOX_LOX) /* use bitwise AND to check if a flag is set. */
		delete_me(); //tell the game to remove yourself
}

//==================cCritterTreasure=============

cCritterTreasure::cCritterTreasure(cGame *pownergame):
cCritter(pownergame)
{
	cPolygon *ppoly = new cPolygon(24);
	ppoly->setFilled(FALSE);
	ppoly->setLineWidthWeight(0.1);
	_collidepriority = cCollider::CP_PLAYER + 1; /* Let this guy call collide on the
		player, as his method is overloaded in a special way. */
	setSprite(ppoly);
	setRadius(cGameBallworld::TREASURERADIUS);
	setFixedflag(TRUE);
	moveTo(cVector(_movebox.hix()- 1.5 * cGameBallworld::TREASURERADIUS,
		_movebox.midy()));
}

int cCritterTreasure::collidesWith(cCritter *pothercritter)
{
	if (pothercritter->IsKindOf(RUNTIME_CLASS(cCritterBallworldPlayer)))
		return cCollider::COLLIDEASCALLER;
	else
		return cCollider::DONTCOLLIDE;
}

BOOL cCritterTreasure::collide(cCritter *pcritter)
{
	if (contains(pcritter)) //disk of pcritter is wholly inside my disk
	{
		playSound("Tada");
		pcritter->addScore(100);
		pcritter->addHealth(1);
		pcritter->moveTo(pgame()->border().locorner());
		return TRUE;
	}
	else
		return FALSE;
}
