#ifndef GAMEINVADER4D
#define GAMEINVADER4D

/*
	The gamestub.* files are meant to be a possible starting point for your games.  Alter the
	gamestub.* files freely.  
	In the code below, we put the word virtual in front of all the methods that we have
	inherited from the base classes and which we plan to overload.  (These methods were
	already declared as virtual in the base classes, so that our overloads can take effect.
	Declaring them virtual in this child class only means that they could be further overloaded 
	in the children of the child.)
*/

#include "game.h"
#include "critterarmed.h"

//=== cGameInvader4D Critters ====================================
class cCritterInvader4DPlayer : public cCritterArmedPlayer //Our player.
{
DECLARE_SERIAL(cCritterInvader4DPlayer);
public:
	cCritterInvader4DPlayer(cGame *pownergame = NULL);
//overloads
	virtual void reset();
	virtual int damage(int hitstrength);
	virtual BOOL collide(cCritter *pcritter); 
	virtual void roll(Real turnangle);
	virtual void update(CPopView *pview);
	virtual void draw(cGraphics *pgraphics, int drawflags, cCritterViewer4D * pviewer4D=NULL);
	virtual cCritterBullet* shoot();
};

class cCritterInvader4DPlayerBullet : public cCritterBullet
{
DECLARE_SERIAL(cCritterInvader4DPlayerBullet);
public:
	cCritterInvader4DPlayerBullet();
	virtual void update(CPopView *pview);
};

class cCritterInvader4DRival : public cCritterArmedRobot //Or just a cCritter if not shooting
{
DECLARE_SERIAL(cCritterInvader4DRival);
public:
	cCritterInvader4DRival(cGame *pownergame = NULL); /* The defualt arg
		means this defines a no-arg constructor as well.*/
//overloads
	virtual int damage(int hitstrength);
	virtual BOOL collide(cCritter *pcritter); 
	virtual void update(CPopView *pview);
};

/* The cCritterBulletSilver only can damage the ptarget of the
	shooter.   */
class cCritterInvader4DRivalBullet : public cCritterBulletSilver
{
DECLARE_SERIAL(cCritterInvader4DRivalBullet);
public:
	cCritterInvader4DRivalBullet();
	virtual void initialize(cCritterArmed *pshooter);
};

class cCritterInvader4DProp : public cCritter
{
DECLARE_SERIAL(cCritterInvader4DProp);
public:
	static Real EXPLOSIONDISPLAYTIME;
	static int FRAGCOUNT;
	cCritterInvader4DProp(cGame *pownergame = NULL); 
//overloads
	virtual int damage(int hitstrength);
	virtual BOOL collide(cCritter *pcritter); 
	virtual void update(CPopView *pview);
	virtual void die(); //Overload to show fragments or explosion bitmap.
};

class cCritterInvader4DPropFrag : public cCritter
{
DECLARE_SERIAL(cCritterInvader4DPropFrag);
public:
	static Real FIXEDLIFETIME;
	cCritterInvader4DPropFrag(cCritter *ppropcritter = NULL); /* Note that this
		constructor takes a prop critter pointer as argument INSTEAD of a game pointer! */
//overloads
	virtual void update(CPopView *pview);
	virtual int collidesWith(cCritter *pcritterother);
};

//======================= cGameInvader4D ================
class cGameInvader4D : public cGame
{
DECLARE_SERIAL(cGameInvader4D);
//Name your statics here
static int PLAYERHEALTH;
static int DEFAULTRIVALCOUNT;
static int DEFAULTSEEDCOUNT;
private:
	int _rivalcount; //possible params, don't have to use these.
public:
	cGameInvader4D();
 //overloads
	virtual void Serialize(CArchive& ar); //Overload for _lastinvasionscore
	virtual void reset();
	virtual void adjustGameParameters();
	virtual CString statusMessage();
	virtual void initializeView(CPopView *pview);
	virtual void initializeViewpoint(cCritterViewer *pviewer); //To change the default view direction.
	virtual void seedCritters();
	virtual void setWrapflag(int wrapflag);
	virtual void step(Real dt, CPopView *pactiveview);
	virtual void drawForeground(cGraphics *pgraphics, int drawflags, cCritterViewer4D *pviewpointcritter4D=NULL);
	BOOL polletSuggestion;
	BOOL showPlayerPosition;
private:
	BOOL gameOver;
	BOOL win;
	int previousLevel;
};

#endif //GAMEINVADER4D