#include "stdafx.h"
#include "hyperpyramid.h"
#include "graphicsopengl.h"
#include "game.h" //Gives include biota.h, critter.h, sprite.h

IMPLEMENT_SERIAL(cSpriteHyperpyramid, cSprite, 0 );
//=================cSpriteHyperpyramid====================
cSpriteHyperpyramid::cSpriteHyperpyramid()
{
	numOfVertices = 9;
	dimension = 4;
	drawnradius = 1.25;//_radius;
	vertices = new double * [dimension];//rows 4
	tmpMatrix = new double * [dimension];
	for(int i=0;i<dimension;i++)
	{	
		vertices[i] = new double[numOfVertices]; //init row by row
		tmpMatrix[i] = new double[numOfVertices]; //init row by row
	}
	
	initializeVertices(drawnradius);

	rotationMatrix = new double * [dimension];//rows 4
	for(i=0;i<dimension;i++)
		rotationMatrix[i] = new double[dimension]; //init row by row
	
	translationMatrix = new double [dimension];

	vertex1 = new cVector4();vertex2 = new cVector4();
	vertex3 = new cVector4(); vertex4 = new cVector4();
	vertex5 = new cVector4(); vertex6 = new cVector4();
	vertex7 = new cVector4(); vertex8 = new cVector4();
	vertex9 = new cVector4(); 

	spinSpeed = 0;
	sinkValue = 0;
}
cSpriteHyperpyramid::~cSpriteHyperpyramid()
{
	for(int i=0;i<dimension;i++)
	{
		delete [] (vertices[i]);
		vertices[i]=NULL;

		delete [] (rotationMatrix[i]);
		rotationMatrix[i]=NULL;

		delete [] (tmpMatrix[i]);
		tmpMatrix[i] = NULL;
	}

	delete [] vertices;
	vertices = NULL;

	delete [] tmpMatrix;
	tmpMatrix = NULL;

	delete [] rotationMatrix;
	rotationMatrix = NULL;

	delete translationMatrix;
	translationMatrix = NULL;

	delete vertex1;delete vertex2;delete vertex3;delete vertex4;
	delete vertex5;delete vertex6;delete vertex7;delete vertex8;
	delete vertex9;
	vertex1 = NULL; vertex2 = NULL; vertex3 = NULL; vertex4 = NULL;
	vertex5 = NULL; vertex6 = NULL; vertex7 = NULL; vertex8 = NULL;
	vertex9 = NULL;
}

void cSpriteHyperpyramid::animate(Real dt, const cVector &critterdirection)
{
	spinSpeed = cRandomizer::pinstance()->randomReal(dt*0.5, dt*2);
	Real angle1 = this->spinSpeed;
	Real angle2 = this->spinSpeed;
	Real angle3 = this->spinSpeed;
	Real angle4 = this->spinSpeed;
	Real angle5 = this->spinSpeed;
	Real angle6 = this->spinSpeed;
	
	_spriteattitude = _spriteattitude* 
		cMatrix4::zRotation(angle4)*
		cMatrix4::yRotation(angle5)*
		cMatrix4::p14Rotation(angle1)*
		cMatrix4::xRotation(angle6)*
		cMatrix4::p24Rotation(angle2)*
		cMatrix4::p34Rotation(angle3);
			
	//extract info on rotation from _attitude and put it
	//in rotationMatrix(4 by 4)
	for(int i=0;i<dimension;i++)
	{
		cVector4 column = _spriteattitude.column(i);
		rotationMatrix[0][i] = column.x();
		rotationMatrix[1][i] = column.y();
		rotationMatrix[2][i] = column.z();
		rotationMatrix[3][i] = column.w();
	}

	//extract info on translation from _attitide and put it
	//in translationMatrix(4 by 1)
	cVector4 column = _spriteattitude.lastColumn();//.column(dimension);
	translationMatrix[0] = column.x();
	translationMatrix[1] = column.y();
	translationMatrix[2] = column.z();
	translationMatrix[3] = column.w();
	
	/*3
		rotationMatrix(4 by 4) times 
		vertices(4 by 16) is another 4 by 16 matrix tmpMatrix
	*/
	for (i=0; i<dimension; i++)
		for (int j=0; j<numOfVertices; j++)
		{
			tmpMatrix[i][j] = 0.0;
			for (int k=0; k<dimension; k++)
			{	
				tmpMatrix[i][j] += rotationMatrix[i][k]*vertices[k][j];
			}
		}
	
	/*3.5
	update tmpMatrix with a 4 by 1 translation matrix
	*/
	for (i=0; i<dimension; i++)
		for (int j=0; j<numOfVertices; j++)
		{
			tmpMatrix[i][j] = translationMatrix[i]+tmpMatrix[i][j];
		}
}

void cSpriteHyperpyramid::draw(cGraphics *pgraphics, int drawflags, cCritterViewer4D * pviewer4D)
{
	BOOL drawfill = filled() && !(drawflags & CPopView::DF_WIREFRAME);

	vertex1->set(tmpMatrix[0][0],tmpMatrix[1][0], tmpMatrix[2][0], tmpMatrix[3][0]);//1st column 
	vertex2->set(tmpMatrix[0][1],tmpMatrix[1][1], tmpMatrix[2][1], tmpMatrix[3][1]);
	vertex3->set(tmpMatrix[0][2],tmpMatrix[1][2], tmpMatrix[2][2], tmpMatrix[3][2]);
	vertex4->set(tmpMatrix[0][3],tmpMatrix[1][3], tmpMatrix[2][3], tmpMatrix[3][3]);
	vertex5->set(tmpMatrix[0][4],tmpMatrix[1][4], tmpMatrix[2][4], tmpMatrix[3][4]);
	vertex6->set(tmpMatrix[0][5],tmpMatrix[1][5], tmpMatrix[2][5], tmpMatrix[3][5]);
	vertex7->set(tmpMatrix[0][6],tmpMatrix[1][6], tmpMatrix[2][6], tmpMatrix[3][6]);
	vertex8->set(tmpMatrix[0][7],tmpMatrix[1][7], tmpMatrix[2][7], tmpMatrix[3][7]);
	vertex9->set(tmpMatrix[0][8],tmpMatrix[1][8], tmpMatrix[2][8], tmpMatrix[3][8]);
		
	if(!drawfill)
	{	
		//front face
		_pcolorstyle->setLineColor(RGB(0,230,150));
		pgraphics->drawline(*vertex1,*vertex2,_pcolorstyle);
		pgraphics->drawline(*vertex2,*vertex3,_pcolorstyle);
		pgraphics->drawline(*vertex3,*vertex4,_pcolorstyle);
		pgraphics->drawline(*vertex4,*vertex1,_pcolorstyle);
			
		//back face
		_pcolorstyle->setLineColor(RGB(0,180,100));
		pgraphics->drawline(*vertex5,*vertex6,_pcolorstyle);
		pgraphics->drawline(*vertex6,*vertex7,_pcolorstyle);
		pgraphics->drawline(*vertex7,*vertex8,_pcolorstyle);
		pgraphics->drawline(*vertex8,*vertex5,_pcolorstyle);

		//left face
		_pcolorstyle->setLineColor(cColorStyle::CN_RED);
		pgraphics->drawline(*vertex1,*vertex5,_pcolorstyle);
		pgraphics->drawline(*vertex4,*vertex8,_pcolorstyle);
		
		//right face
		_pcolorstyle->setLineColor(cColorStyle::CN_YELLOW);
		pgraphics->drawline(*vertex2,*vertex6,_pcolorstyle);
		pgraphics->drawline(*vertex3,*vertex7,_pcolorstyle);
			
		//prism tip
		_pcolorstyle->setLineColor(cColorStyle::CN_BLUE);
		pgraphics->drawline(*vertex9,*vertex1,_pcolorstyle);
		pgraphics->drawline(*vertex9,*vertex2,_pcolorstyle);
		pgraphics->drawline(*vertex9,*vertex5,_pcolorstyle);
		pgraphics->drawline(*vertex9,*vertex6,_pcolorstyle);
		pgraphics->drawline(*vertex9,*vertex3,_pcolorstyle);
		pgraphics->drawline(*vertex9,*vertex4,_pcolorstyle);
		pgraphics->drawline(*vertex9,*vertex7,_pcolorstyle);
		pgraphics->drawline(*vertex9,*vertex8,_pcolorstyle);

	}
	if(drawfill)//to be changed later to 8 faces
	{	
		
		//front face
		_pcolorstyle->setFillColor(RGB(0,230,150));
		pgraphics->drawrectangle(*vertex1, *vertex2,*vertex3,*vertex4, _pcolorstyle);
		
		//back face
		_pcolorstyle->setFillColor(RGB(0,180,100));
		pgraphics->drawrectangle(*vertex5, *vertex6,*vertex7, *vertex8,_pcolorstyle);

		//left face
		_pcolorstyle->setFillColor(cColorStyle::CN_RED);
		pgraphics->drawrectangle(*vertex1, *vertex5,*vertex8, *vertex4,_pcolorstyle);

		//right face
		_pcolorstyle->setFillColor(cColorStyle::CN_YELLOW);
		pgraphics->drawrectangle(*vertex2, *vertex6,*vertex7, *vertex3,_pcolorstyle);

		//top face
		_pcolorstyle->setFillColor(cColorStyle::CN_PURPLE);
		pgraphics->drawrectangle(*vertex1, *vertex5,*vertex6, *vertex2,_pcolorstyle);

		//bottom face
		_pcolorstyle->setFillColor(cColorStyle::CN_ORANGE);
		pgraphics->drawrectangle(*vertex4, *vertex8,*vertex7, *vertex3,_pcolorstyle);

		//pyramid tip
		_pcolorstyle->setLineColor(cColorStyle::CN_BLUE);
		_pcolorstyle->setFillColor(cColorStyle::CN_LIGHTBLUE);
		pgraphics->drawtriangle(*vertex9, *vertex1,*vertex2,_pcolorstyle);
		pgraphics->drawtriangle(*vertex9, *vertex2,*vertex6,_pcolorstyle);
		pgraphics->drawtriangle(*vertex9, *vertex6,*vertex5,_pcolorstyle);
		pgraphics->drawtriangle(*vertex9, *vertex5,*vertex1,_pcolorstyle);
		
//		_pcolorstyle->setFillColor(cColorStyle::CN_LIGHTGRAY);
		pgraphics->drawtriangle(*vertex9, *vertex4,*vertex3,_pcolorstyle);
		pgraphics->drawtriangle(*vertex9, *vertex3,*vertex7,_pcolorstyle);
		pgraphics->drawtriangle(*vertex9, *vertex7,*vertex8,_pcolorstyle);
		pgraphics->drawtriangle(*vertex9, *vertex8,*vertex4,_pcolorstyle);

		//fill up the gaps in between
//		_pcolorstyle->setFillColor(cColorStyle::CN_LIGHTGRAY);
		pgraphics->drawtriangle(*vertex9, *vertex1,*vertex3,_pcolorstyle);
		pgraphics->drawtriangle(*vertex9, *vertex5,*vertex7,_pcolorstyle);
		pgraphics->drawtriangle(*vertex9, *vertex2,*vertex4,_pcolorstyle);
		pgraphics->drawtriangle(*vertex9, *vertex6,*vertex8,_pcolorstyle);
	}
}

void cSpriteHyperpyramid::initializeVertices(Real radius)
{
	sinkValue = 0.3;
	//outter cube
	//front face
	//vertex 1 
	vertices[0][0] = -radius;
	vertices[1][0] = radius;//0;
	vertices[2][0] = radius;
	vertices[3][0] = 0-sinkValue;//radius;
	//vertex 2 
	vertices[0][1] = radius;
	vertices[1][1] = radius;//0;
	vertices[2][1] = radius;
	vertices[3][1] = 0-sinkValue;//radius;
	//vertex 3 w/ negative w
	vertices[0][2] = radius;
	vertices[1][2] = -radius;
	vertices[2][2] = radius;
	vertices[3][2] = 0-sinkValue;//-radius;
	//vertex 4 w/ negative w
	vertices[0][3] = -radius;
	vertices[1][3] = -radius;
	vertices[2][3] = radius;
	vertices[3][3] = 0-sinkValue;//-radius;
	
	//back face
	//vertex 5 
	vertices[0][4] = -radius;
	vertices[1][4] = radius;//0;
	vertices[2][4] = -radius;
	vertices[3][4] = 0-sinkValue;//radius;
	//vertex 6 
	vertices[0][5] = radius;
	vertices[1][5] = radius;//0;
	vertices[2][5] = -radius;
	vertices[3][5] = 0-sinkValue;//radius;
	//vertex 7 w/ negative w
	vertices[0][6] = radius;
	vertices[1][6] = -radius;
	vertices[2][6] = -radius;
	vertices[3][6] = 0-sinkValue;//-radius;
	//vertex 8
	vertices[0][7] = -radius;
	vertices[1][7] = -radius;
	vertices[2][7] = -radius;
	vertices[3][7] = 0-sinkValue;//-radius;
	
	//vertex 9 pyramid tip
	vertices[0][8] = 0;//radius;
	vertices[1][8] = radius*2;
	vertices[2][8] = 0;//radius;
	vertices[3][8] = radius*2-sinkValue;//0;//radius*2;
}

void cSpriteHyperpyramid::setRadius(Real radius)
{
	cSprite::setRadius(radius);
	
	//The actual critter's size is about 2/3 of a 
	//sprite hypercube.  To increase hit probability
	//we reduce the actual radius
	drawnradius = radius*0.7;//0.66;
	initializeVertices(drawnradius);	

}