#include "stdafx.h"
#include "graphicsopengl.h"
#include "hypertetrahedron.h"
#include "game.h" //Gives include biota.h, critter.h, sprite.h
IMPLEMENT_SERIAL(cSpriteHypertetrahedron, cSprite, 0 );
//=================cSpriteHypertetrahedron Code=================================
cSpriteHypertetrahedron::cSpriteHypertetrahedron()
{
	numOfVertices = 5;
	dimension = 4;
	drawnradius = 1.25;//_radius;
	vertices = new double * [dimension];//rows 4
	tmpMatrix = new double * [dimension];
	for(int i=0;i<dimension;i++)
	{	
		vertices[i] = new double[numOfVertices]; //init row by row
		tmpMatrix[i] = new double[numOfVertices]; //init row by row
	}
	
	initializeVertices(drawnradius);

	rotationMatrix = new double * [dimension];//rows 4
	for(i=0;i<dimension;i++)
		rotationMatrix[i] = new double[dimension]; //init row by row
	
	translationMatrix = new double [dimension];

	vertex1 = new cVector4();vertex2 = new cVector4();
	vertex3 = new cVector4(); vertex4 = new cVector4();
	vertex5 = new cVector4(); 

	spinSpeed = 0;
	sinkValue = 0.0;
}

cSpriteHypertetrahedron::~cSpriteHypertetrahedron()
{
	for(int i=0;i<dimension;i++)
	{
		delete [] (vertices[i]);
		vertices[i]=NULL;

		delete [] (rotationMatrix[i]);
		rotationMatrix[i]=NULL;

		delete [] (tmpMatrix[i]);
		tmpMatrix[i] = NULL;
	}

	delete [] vertices;
	vertices = NULL;

	delete [] tmpMatrix;
	tmpMatrix = NULL;

	delete [] rotationMatrix;
	rotationMatrix = NULL;

	delete translationMatrix;
	translationMatrix = NULL;

	delete vertex1;delete vertex2;delete vertex3;delete vertex4;
	delete vertex5;
	vertex1 = NULL; vertex2 = NULL; vertex3 = NULL; vertex4 = NULL;
	vertex5 = NULL;
}

void cSpriteHypertetrahedron::animate(Real dt, const cVector &critterdirection)
{
	spinSpeed = cRandomizer::pinstance()->randomReal(dt*0.5, dt*2);//0.05
	
	_spriteattitude = _spriteattitude* 
		cMatrix4::zRotation(spinSpeed)*
		cMatrix4::yRotation(spinSpeed)*
		cMatrix4::p14Rotation(spinSpeed)*
		cMatrix4::xRotation(spinSpeed)*
		cMatrix4::p24Rotation(spinSpeed)*
		cMatrix4::p34Rotation(spinSpeed);
			
	//extract info on rotation from _attitude and put it
	//in rotationMatrix(4 by 4)
	for(int i=0;i<dimension;i++)
	{
		cVector4 column = _spriteattitude.column(i);
		rotationMatrix[0][i] = column.x();
		rotationMatrix[1][i] = column.y();
		rotationMatrix[2][i] = column.z();
		rotationMatrix[3][i] = column.w();
	}

	//extract info on translation from _attitide and put it
	//in translationMatrix(4 by 1)
	cVector4 column = _spriteattitude.lastColumn();//.column(dimension);
	translationMatrix[0] = column.x();
	translationMatrix[1] = column.y();
	translationMatrix[2] = column.z();
	translationMatrix[3] = column.w();

	
	/*3
		rotationMatrix(4 by 4) times 
		vertices(4 by 16) is another 4 by 16 matrix tmpMatrix
	*/
	for (i=0; i<dimension; i++)
		for (int j=0; j<numOfVertices; j++)
		{
			tmpMatrix[i][j] = 0.0;
			for (int k=0; k<dimension; k++)
				tmpMatrix[i][j] += rotationMatrix[i][k]*vertices[k][j];
		}
	
	/*3.5
	update tmpMatrix with a 4 by 1 translation matrix
	*/
	for (i=0; i<dimension; i++)
		for (int j=0; j<numOfVertices; j++)
			tmpMatrix[i][j] = translationMatrix[i]+tmpMatrix[i][j];
}

void cSpriteHypertetrahedron::draw(cGraphics *pgraphics, int drawflags, cCritterViewer4D * pviewer4D)
{
	BOOL drawfill = filled() && !(drawflags & CPopView::DF_WIREFRAME);

	vertex1->set(tmpMatrix[0][0],tmpMatrix[1][0], tmpMatrix[2][0], tmpMatrix[3][0]);//1st column 
	vertex2->set(tmpMatrix[0][1],tmpMatrix[1][1], tmpMatrix[2][1], tmpMatrix[3][1]);
	vertex3->set(tmpMatrix[0][2],tmpMatrix[1][2], tmpMatrix[2][2], tmpMatrix[3][2]);
	vertex4->set(tmpMatrix[0][3],tmpMatrix[1][3], tmpMatrix[2][3], tmpMatrix[3][3]);
	vertex5->set(tmpMatrix[0][4],tmpMatrix[1][4], tmpMatrix[2][4], tmpMatrix[3][4]);
		
	if(!drawfill)
	{
		_pcolorstyle->setLineColor(RGB(0,230,150));
		pgraphics->drawline(*vertex1,*vertex2,_pcolorstyle);
		pgraphics->drawline(*vertex3,*vertex4,_pcolorstyle);
		pgraphics->drawline(*vertex1,*vertex3,_pcolorstyle);
		pgraphics->drawline(*vertex1,*vertex4,_pcolorstyle);
		pgraphics->drawline(*vertex2,*vertex3,_pcolorstyle);
		pgraphics->drawline(*vertex2,*vertex4,_pcolorstyle);

		//4D
		_pcolorstyle->setLineColor(cColorStyle::CN_RED);
		pgraphics->drawline(*vertex5,*vertex1,_pcolorstyle);
		pgraphics->drawline(*vertex5,*vertex2,_pcolorstyle);
		pgraphics->drawline(*vertex5,*vertex3,_pcolorstyle);
		pgraphics->drawline(*vertex5,*vertex4,_pcolorstyle);
	}
	if(drawfill)
	{	
		_pcolorstyle->setLineColor(RGB(0,230,150));
		_pcolorstyle->setFillColor(cColorStyle::CN_BLUE);
		pgraphics->drawtriangle(*vertex1,*vertex2,*vertex3,_pcolorstyle);
		
		_pcolorstyle->setFillColor(cColorStyle::CN_GREEN);
		pgraphics->drawtriangle(*vertex1,*vertex2,*vertex4,_pcolorstyle);

		_pcolorstyle->setFillColor(cColorStyle::CN_ORANGE);
		pgraphics->drawtriangle(*vertex1,*vertex3,*vertex4,_pcolorstyle);

		_pcolorstyle->setFillColor(cColorStyle::CN_PURPLE);
		pgraphics->drawtriangle(*vertex2,*vertex3,*vertex4,_pcolorstyle);

		//4D
		_pcolorstyle->setLineColor(cColorStyle::CN_RED);
		_pcolorstyle->setFillColor(cColorStyle::CN_LIGHTBLUE);
		pgraphics->drawtriangle(*vertex5,*vertex1,*vertex2,_pcolorstyle);

		_pcolorstyle->setFillColor(cColorStyle::CN_LIGHTGRAY);
		pgraphics->drawtriangle(*vertex5,*vertex1,*vertex3,_pcolorstyle);

		_pcolorstyle->setFillColor(cColorStyle::CN_LIGHTGREEN);
		pgraphics->drawtriangle(*vertex5,*vertex1,*vertex4,_pcolorstyle);

		_pcolorstyle->setFillColor(cColorStyle::CN_LIGHTPURPLE);
		pgraphics->drawtriangle(*vertex5,*vertex2,*vertex3,_pcolorstyle);

		_pcolorstyle->setFillColor(cColorStyle::CN_LIGHTRED);
		pgraphics->drawtriangle(*vertex5,*vertex2,*vertex4,_pcolorstyle);

		_pcolorstyle->setFillColor(cColorStyle::CN_LIGHTYELLOW);
		pgraphics->drawtriangle(*vertex5,*vertex3,*vertex4,_pcolorstyle);
	}
	
}

void cSpriteHypertetrahedron::initializeVertices(Real radius)
{
	sinkValue = 0.3;
	//vertex 1 
	vertices[0][0] = -radius;
	vertices[1][0] = 0;
	vertices[2][0] = radius;
	vertices[3][0] = 0-sinkValue;//radius;
	//vertex 2 
	vertices[0][1] = radius;
	vertices[1][1] = 0;
	vertices[2][1] = radius;
	vertices[3][1] = 0-sinkValue;//radius;
	//vertex 3
	vertices[0][2] = 0;
	vertices[1][2] = -radius;
	vertices[2][2] = -radius;
	vertices[3][2] = 0-sinkValue;//-radius;
	//vertex 4
	vertices[0][3] = 0;
	vertices[1][3] = radius;
	vertices[2][3] = -radius;
	vertices[3][3] = 0-sinkValue;//-radius;
	
	//vertex 5 
	vertices[0][4] = 0;
	vertices[1][4] = 0;
	vertices[2][4] = 0;
	vertices[3][4] = radius-sinkValue;
}

void cSpriteHypertetrahedron::setRadius(Real radius)
{
	cSprite::setRadius(radius);
	//The actual critter's size is about 2/3 of a 
	//sprite hypercube.  To increase hit probability
	//we reduce the actual radius
	drawnradius = radius;//0.66;
	initializeVertices(drawnradius);	
}
