

#ifndef BOX_HPP
#define BOX_HPP

#include "vectortransformation.h" //For cVector2 and cVector3
#include "colorstyle.h"
class cGraphics; //So we can use a cGraphics* pointer in the draw method declarations.
class cCritterViewer4D;
class cRealBox2;
class cRealBox3;
class cRealBox4; //Wyley
//Wyley
#ifdef TWODVECTORS
	typedef class cRealBox2 cRealBox;
#endif
#ifdef THREEDVECTORS
	typedef class cRealBox3 cRealBox;
#define DRAW3DBOX
#endif
#ifdef FOURDVECTORS
	typedef class cRealBox4 cRealBox;
#define DRAW4DBOX
//#define DRAW3DBOX
	//typedef class cRealBox3 cRealBox;
#endif 


//===========
/* Outcodes for cRealBox2::outcode and for cCritterWall::outcode().  These can't be const
static int values because then I can't use them in switch statements. */
//???
#define BOX_INSIDE 0
#define BOX_LOX 1
#define BOX_HIX 2
#define BOX_LOY 4
#define BOX_HIY 8
#define BOX_LOZ 16
#define BOX_HIZ 32
#define BOX_LOW	64
#define BOX_HIW 128
#define BOX_HIY_LOX (BOX_HIY|BOX_LOX)
#define BOX_HIY_HIX (BOX_HIY|BOX_HIX)
#define BOX_LOY_LOX (BOX_LOY|BOX_LOX)
#define BOX_LOY_HIX (BOX_LOY|BOX_HIX)
//Wyley
#define BOX_HIY_LOW (BOX_HIY|BOX_LOW)
#define BOX_HIY_HIW (BOX_HIY|BOX_HIW)
#define BOX_LOY_LOW (BOX_LOY|BOX_LOW)
#define BOX_LOY_HIW (BOX_LOY|BOX_HIW)
#define BOX_HIX_LOW (BOX_HIX|BOX_LOW)
#define BOX_HIX_HIW (BOX_HIX|BOX_HIW)
#define BOX_LOX_LOW (BOX_LOX|BOX_LOW)
#define BOX_LOX_HIW (BOX_LOX|BOX_HIW)

#define BOX_ALL (BOX_LOY_HIX|BOX_HIY_LOX)
#define BOX_X (BOX_LOX | BOX_HIX)
#define BOX_Y (BOX_LOY | BOX_HIY)
#define BOX_Z (BOX_LOZ | BOX_HIZ)
#define BOX_W (BOX_LOW | BOX_HIW)
//==============
class cRealBox2 : public CObject
{
DECLARE_SERIAL(cRealBox2);
protected:
	Real _lox, _hix, _loy, _hiy;
	cColorStyle *_pcolorstyle; //In case you want to draw it.
    void _arrange(); //helper function
	void _initialize(Real px, Real py, Real qx, Real qy); /* Make this private and use a more
		dimension-independnt form as the public method*/
	void _copy(const cRealBox2 &box);
	void _copy(const cRealBox3 &box);
public:
	//constructors
	 /* Makes a box of these dimensions centered on the origin. We give the default box a
		typical 4:3 screen aspect , as is seen in 800x600 pixel resolution. 
		Often this won't fill a typical window nicely, what with all the
		menu, tool, and status bars, so we change it at	the program start.*/
	cRealBox2(Real xsize = 4.0, Real ysize = 3.0, Real dummyzsize = 0.0); 
	cRealBox2(const cVector2 &leftlocorner, const cVector2 &righthicorner);
	cRealBox2(const cVector2 &center, Real xsize, Real ysize, 
		Real dummyzsize = 0.0); 
	cRealBox2(const cVector2 &center, Real edge); //square
	cRealBox2(const cRealBox2 &box):_pcolorstyle(NULL){_copy(box);}
	cRealBox2(const cRealBox3 &box):_pcolorstyle(NULL){_copy(box);}
	cRealBox2& operator=(const cRealBox2 &box){_copy(box); return *this;}
	void set(const cVector2 &locorner, const cVector2 &hicorner);
	void set(const cVector2 &center, Real xsize, Real ysize, Real zsize=0.0);
	void set(Real xsize, Real ysize, Real zsize=0.0); /* Have the dummy zsize arg in these constructors
		for consistency with the cRealBox3 interface. */
	virtual ~cRealBox2(){delete _pcolorstyle;}
	//mutators
	void moveTo(const cVector2 &newcenter){set(newcenter, xsize(), ysize());}
	void matchAspect(int cx, int cy);
	void setZRange(Real loz, Real hiz){}; //does nothing, just here to match cRealBox3
	//accessors
	Real lox()const {return _lox;}
	Real hix()const {return _hix;}
	Real loy()const {return _loy;}
	Real hiy()const {return _hiy;}
	Real loz()const {return 0.0;}
	Real hiz()const {return 0.0;}
	Real midx()const {return (_hix + _lox) / 2.0;}
	Real midy()const {return (_hiy + _loy) / 2.0;}
	Real midz()const{return 0.0;}
	cVector2 center()const{return cVector2((_lox + _hix) / 2.0, (_loy + _hiy) / 2.0);}
	cVector2 locorner()const{return cVector2(_lox, _loy);}
	cVector2 hicorner()const{return cVector2(_hix, _hiy);}
	cVector2 corner(int i)const; //Method for listing corners 0 to 3 when you need to check all.
	Real xsize()const {return _hix - _lox;}
	Real ysize()const {return _hiy - _loy;}
	Real zsize()const{return 0.0;}
	Real minsize()const {return __min(xsize(), ysize());}
	Real maxsize()const {return __max(xsize(), ysize());} /* __min or __max with
		two underscores is a C macro */ 
	Real xradius()const {return (_hix - _lox)/2.0;}
	Real yradius()const {return (_hiy - _loy)/2.0;}
	Real zradius()const{return 0.0;}
	Real radius()const {return sqrt(xradius()*xradius() + yradius()*yradius());}
	cRealBox2 innerBox(Real radius)const; /* Return a box offset inward by radius
		distance from each wall.  We clamp radius be less than half the size() */
	cRealBox2 outerBox(Real radius)const; /* Return a box offset outward by radius */
	//methods
	cVector2 randomVector()const;
	BOOL inside(const cVector2 &testpos)const;
	int outcode(const cVector2 &testpos)const; //Outcodes are defined in realbox.h
	Real distanceTo(const cVector2 &testpos)const;
	Real maxDistanceToCorner(const cVector2 &testpos)const;
	int addBounce(cVector2 &position, cVector2 &velocity, Real bounciness)const;
     /*add velocity to position, but if you pass the border, then
     	reflect velocity and get a reflected position. */
	int addBounce(cVector2 &position, cVector2 &velocity, Real bounciness, Real dt)const;
		/* add dt*velocity to position, and do the bounce to velocity the same. */
	int wrap(cVector2 &position)const; /* If you move off one edge,
		then come back in the same amount from the other side */
	int wrap(cVector2 &position, cVector2 &wrapposition1, cVector2 &wrapposition2,
		cVector2 &wrapposition3, Real radius) const; /* If you are within
		radius of an edge, put the possible wrap values in wrappositions. */
	int clamp(cVector2 &position)const; /*Make sure position is inside*/
	int clamp(cVector2 &position, cVector2 &velocity)const; /*Make sure position is inside
		and kill any velocity in a wall-hitting direction. */
//	CRect realToPixel(const cRealPixelConverter &crealpixelconverter)const;
	virtual	void draw(cGraphics *pgraphics, int drawflags,cCritterViewer4D * pviewer4D = NULL);
	friend cRealBox2 operator*(Real scale, const cRealBox2 &box); /* Return
		a box scale times as big as box. */
	friend BOOL operator==(const cRealBox2 arect, const cRealBox2 brect);
	friend BOOL  operator!=(const cRealBox2 arect, const cRealBox2 brect);
//Graphics methods
	cColorStyle* pcolorstyle()const{return _pcolorstyle;}
//Serialize methods
	virtual void Serialize(CArchive &ar); /* Be sure to delete _pcolorstyle before reading in,
		or you have a memory leak. */
	friend CArchive& operator<<(CArchive& ar, cRealBox2 &realbox);
	friend CArchive& operator>>(CArchive& ar, cRealBox2 &realbox);
};

//3D=================================================================

class cRealBox3 : public CObject
{
DECLARE_SERIAL(cRealBox3);
//protected:
//Wyley
public:
	Real _lox, _hix, _loy, _hiy, _loz, _hiz;
	cColorStyle *_pcolorstyle;
    void _arrange(); //helper function
	void _initialize(Real px, Real py, Real pz, Real qx, Real qy, Real qz); /* Make this private and use 
		a more dimension-independnt form as the public method*/
	void _copy(const cRealBox2 &box);
	void _copy(const cRealBox3 &box);
	//Wyley
	void _copy(const cRealBox4 &box);
public:
	//constructors
	cRealBox3(Real xsize = 4.0, Real ysize = 3.0, Real zsize = 0.0); 
	cRealBox3(const cVector3 &locorner, const cVector3 &hicorner);
	//Wyley
	cRealBox3(const cVector4 &locorner, const cVector4 &hicorner);
	
	cRealBox3(const cVector3 &center, Real xsize, Real ysize, 
		Real zsize = 0.0); 
	//Wyley
	cRealBox3(const cVector4 &center, Real xsize, Real ysize, 
		Real zsize = 0.0);
	
	cRealBox3(const cVector3 &center, Real edge); //cube
	//Wyley
	cRealBox3(const cVector4 &center, Real edge);

	cRealBox3(const cRealBox2 &box):_pcolorstyle(NULL){_copy(box);}
	cRealBox3(const cRealBox3 &box):_pcolorstyle(NULL){_copy(box);}
	//Wyley
	cRealBox3(const cRealBox4 &box):_pcolorstyle(NULL){_copy(box);}
	cRealBox3& operator=(const cRealBox3 &box){_copy(box); return *this;}
	
	void set(const cVector3 &locorner, const cVector3 &hicorner);
	//Wyley
	void set(const cVector4 &locorner, const cVector4 &hicorner);
	
	void set(const cVector3 &center, Real xsize, Real ysize, Real zsize=0.0);
	//Wyley
	void set(const cVector4 &center, Real xsize, Real ysize, Real zsize=0.0);

	void set(Real xsize, Real ysize, Real zsize=0.0); /* Have the dummy zsize arg in these constructors
		for consistency with the cRealBox3 interface. */
	virtual ~cRealBox3(){delete _pcolorstyle;}
	//accessors
	Real lox()const {return _lox;}
	Real hix()const {return _hix;}
	Real loy()const {return _loy;}
	Real hiy()const {return _hiy;}
	Real loz()const {return _loz;}
	Real hiz()const {return _hiz;}
	//cVector3 locorner()const{return cVector3(_lox, _loy, _loz);}
	//cVector3 hicorner()const{return cVector3(_hix, _hiy, _hiz);}
	//Wyley, replace above so cRealBox3 works with 3D and 4D vectors
	cVector locorner()const;//{return cVector4(_lox, _loy, _loz, 0.0);}
	cVector hicorner()const;//{return cVector4(_hix, _hiy, _hiz, 0.0);}

	Real midx()const {return (_hix + _lox) / 2.0;}
	Real midy()const {return (_hiy + _loy) / 2.0;}
	Real midz()const {return (_hiz + _loz) / 2.0;}
	cVector3 center()const{return cVector3(midx(), midy(), midz());}
	cVector3 corner(int i)const; //Method for listing corners 0 to 7 when you need to check all.
	Real xsize()const {return _hix - _lox;}
	Real ysize()const {return _hiy - _loy;}
	Real zsize()const {return _hiz - _loz;}
	Real minsize()const; //Return smallest non-zero dimension.
	Real maxsize()const {return __max(__max(xsize(), ysize()), zsize());} /* __min or __max with
		two underscores is a C macro */ 
	Real xradius()const {return (_hix - _lox)/2.0;}
	Real yradius()const {return (_hiy - _loy)/2.0;}
	Real zradius()const {return (_hiz - _loz)/2.0;}
	Real radius()const {return sqrt(xradius()*xradius() + yradius()*yradius()+ zradius()*zradius());}
	cRealBox3 innerBox(Real radius)const; /* Return a box offset inward by radius
		distance from each wall, though we don't offset more than will fit. */
	cRealBox3 outerBox(Real radius)const; /* Return a box offset outward by radius. */
	//mutators
	void matchAspect(int cx, int cy);
	void setZRange(Real loz, Real hiz){ASSERT(loz<=hiz); _loz = loz; _hiz = hiz;} 
		/* Convenient for making a 2D box thick. */
	//methods
	cVector3 randomVector()const;
	BOOL inside(const cVector3 &testpos)const;
	int outcode(const cVector3 &testpos)const; //Outcodes are defined in realbox.h
	Real distanceTo(const cVector3 &testpos)const;
	Real maxDistanceToCorner(const cVector3 &testpos)const;
	int addBounce(cVector3 &position, cVector3 &velocity, Real bounciness)const;
     /*add velocity to position, but if you pass the border, then
     	reflect velocity and get a reflected position. */
	int addBounce(cVector3 &position, cVector3 &velocity, Real bounciness, Real dt)const;
		/* add dt*velocity to position, and do the bounce to velocity the same. */
	//Wyley
	int addBounce(cVector4 &position, cVector4 &velocity, Real bounciness)const;
    int addBounce(cVector4 &position, cVector4 &velocity, Real bounciness, Real dt)const;
	
	int wrap(cVector3 &position)const; /* If you move off one edge,
		then come back in the same amount from the other side */
	int wrap(cVector3 &position, cVector3 &wrapposition1, cVector3 &wrapposition2,
		cVector3 &wrapposition3, Real radius) const; /* If you are within
		radius of an edge, put the possible wrap values in wrappositions. */
	//Wyley
	int wrap(cVector4 &position)const;
	int wrap(cVector4 &position, cVector4 &wrapposition1, cVector4 &wrapposition2,
		cVector4 &wrapposition3, Real radius)const;
	
	int clamp(cVector3 &position)const; /*Make sure position is inside*/
	int clamp(cVector3 &position, cVector3 &velocity)const; /*Make sure position is inside
		and kill any velocity in a wall-hitting direction. */

	//Wyley
	int clamp(cVector4 &position)const;
	int clamp(cVector4 &position, cVector4 &velocity)const;

//	CRect realToPixel(const cRealPixelConverter &crealpixelconverter);
	virtual	void draw(cGraphics *pgraphics, int drawflags,cCritterViewer4D * pviewer4D = NULL);
	friend cRealBox3 operator*(Real scale, const cRealBox3 &box); /* Return
		a box scale times as big as box. */
	friend BOOL operator==(const cRealBox3 arect, const cRealBox3 brect);
	friend BOOL operator!=(const cRealBox3 arect, const cRealBox3 brect);
//Graphics methods
	cColorStyle* pcolorstyle()const{return _pcolorstyle;}
//Serialize methods
	virtual void Serialize(CArchive &ar); /* Be sure to delete _pcolorstyle before reading in,
		or you have a memory leak. */
	friend CArchive& operator<<(CArchive& ar, cRealBox3 &realbox);
	friend CArchive& operator>>(CArchive& ar, cRealBox3 &realbox);
};

//========================CRealBox4 Wyley ======================
class cRealBox4 : public CObject
{
DECLARE_SERIAL(cRealBox4);
protected:
	Real _lox, _hix, _loy, _hiy, _loz, _hiz, _low, _hiw;
	cColorStyle *_pcolorstyle;
    void _arrange(); //helper function
	
	void _initialize(Real px, Real py, Real pz, Real pw, Real qx, Real qy, Real qz, Real qw); /* Make this private and use 
		a more dimension-independnt form as the public method*/
	void _copy(const cRealBox2 &box);
	void _copy(const cRealBox3 &box);
	void _copy(const cRealBox4 &box);
public:
	//constructors
	cRealBox4(Real xsize = 4.0, Real ysize = 3.0, Real zsize = 0.0, Real wsize = 0.0); 
	cRealBox4(const cVector4 &locorner, const cVector4 &hicorner);
	cRealBox4(const cVector4 &center, Real xsize, Real ysize, 
		Real zsize = 0.0, Real wsize = 0.0); 
	cRealBox4(const cVector4 &center, Real edge); 
	cRealBox4(const cRealBox2 &box):_pcolorstyle(NULL){_copy(box);}
	cRealBox4(const cRealBox4 &box):_pcolorstyle(NULL){_copy(box);}
	cRealBox4& operator=(const cRealBox4 &box){_copy(box); return *this;}
	void set(const cVector4 &locorner, const cVector4 &hicorner);
	void set(const cVector4 &center, Real xsize, Real ysize, Real zsize=0.0, Real wsize=0.0);
	void set(Real xsize, Real ysize, Real zsize=0.0, Real wsize=0.0); /* Have the dummy zsize arg in these constructors
		for consistency with the cRealBox4 interface. */
	virtual ~cRealBox4(){delete _pcolorstyle;}
	//accessors
	Real lox()const {return _lox;}
	Real hix()const {return _hix;}
	Real loy()const {return _loy;}
	Real hiy()const {return _hiy;}
	Real loz()const {return _loz;}
	Real hiz()const {return _hiz;}
	Real low()const {return _low;}
	Real hiw()const {return _hiw;}
	//cVector3 locorner()const{return cVector3(_lox, _loy, _loz);}
	//cVector3 hicorner()const{return cVector3(_hix, _hiy, _hiz);}
	cVector4 locorner()const{return cVector4(_lox, _loy, _loz, _low);}
	cVector4 hicorner()const{return cVector4(_hix, _hiy, _hiz, _hiw);}
	Real midx()const {return (_hix + _lox) / 2.0;}
	Real midy()const {return (_hiy + _loy) / 2.0;}
	Real midz()const {return (_hiz + _loz) / 2.0;}
	Real midw()const {return (_hiw + _low) / 2.0;}
	//cVector3 center()const{return cVector3(midx(), midy(), midz());}
	cVector4 center()const{return cVector4(midx(), midy(), midz(), midw());}
	//cVector3 corner(int i)const; //Method for listing corners 0 to 7 when you need to check all.
	//???cVector4
	cVector4 corner(int i)const;
	Real xsize()const {return _hix - _lox;}
	Real ysize()const {return _hiy - _loy;}
	Real zsize()const {return _hiz - _loz;}
	Real wsize()const {return _hiw - _low;}
	//???
	Real minsize()const; //Return smallest non-zero dimension.
	//???
	Real maxsize()const {return __max(__max(xsize(), ysize()), zsize());} /* __min or __max with
		two underscores is a C macro */ 
	Real xradius()const {return (_hix - _lox)/2.0;}
	Real yradius()const {return (_hiy - _loy)/2.0;}
	Real zradius()const {return (_hiz - _loz)/2.0;}
	Real wradius()const {return (_hiw - _low)/2.0;}
	Real radius()const {return sqrt(xradius()*xradius() + yradius()*yradius()+ zradius()*zradius()+wradius()*wradius());}
	//???cRealBox4
	cRealBox4 innerBox(Real radius)const; /* Return a box offset inward by radius
		distance from each wall, though we don't offset more than will fit. */
	//???cRealBox4
	cRealBox4 outerBox(Real radius)const; /* Return a box offset outward by radius. */
	//mutators
	//???
	void matchAspect(int cx, int cy);
	//Wyley
	void matchAspect(int cx, int cy, int cz);
	//???
	void setZRange(Real loz, Real hiz){ASSERT(loz<=hiz); _loz = loz; _hiz = hiz;} 
		/* Convenient for making a 2D box thick. */
	//methods
	//cVector3 randomVector()const;
	//???
	cVector4 randomVector()const;
	//???
	BOOL inside(const cVector4 &testpos)const;
	//???
	int outcode(const cVector4 &testpos)const; //Outcodes are defined in realbox.h
	//???
	Real distanceTo(const cVector4 &testpos)const;
	//???
	Real maxDistanceToCorner(const cVector4 &testpos)const;
	//???
	int addBounce(cVector4 &position, cVector4 &velocity, Real bounciness)const;
     /*add velocity to position, but if you pass the border, then
     	reflect velocity and get a reflected position. */
	//???
	int addBounce(cVector4 &position, cVector4 &velocity, Real bounciness, Real dt)const;
		/* add dt*velocity to position, and do the bounce to velocity the same. */
	//???
	int wrap(cVector4 &position)const; /* If you move off one edge,
		then come back in the same amount from the other side */
	//???
	int wrap(cVector4 &position, cVector4 &wrapposition1, cVector4 &wrapposition2,
		cVector4 &wrapposition4, Real radius) const; /* If you are within
		radius of an edge, put the possible wrap values in wrappositions. */
	//???
	int clamp(cVector4 &position)const; /*Make sure position is inside*/
	//???
	int clamp(cVector4 &position, cVector4 &velocity)const; /*Make sure position is inside
		and kill any velocity in a wall-hitting direction. */
//	CRect realToPixel(const cRealPixelConverter &crealpixelconverter);
	//???
	virtual	void draw(cGraphics *pgraphics, int drawflags,cCritterViewer4D * pviewer4D = NULL);
	friend cRealBox4 operator*(Real scale, const cRealBox4 &box); /* Return
		a box scale times as big as box. */
	//???
	friend BOOL operator==(const cRealBox4 arect, const cRealBox4 brect);
	friend BOOL operator!=(const cRealBox4 arect, const cRealBox4 brect);
//Graphics methods
	cColorStyle* pcolorstyle()const{return _pcolorstyle;}
//Serialize methods
	virtual void Serialize(CArchive &ar); /* Be sure to delete _pcolorstyle before reading in,
		or you have a memory leak. */
	friend CArchive& operator<<(CArchive& ar, cRealBox4 &realbox);
	friend CArchive& operator>>(CArchive& ar, cRealBox4 &realbox);
};


#endif //BOX_HPP
